@@ -1,7 +1,12 @@
 #!/usr/bin/perl
-#$Id: flood.pl 531 2010-01-11 00:40:18Z pro $ $URL: svn://svn.setun.net/dcppp/trunk/examples/flood.pl $
+#$Id: flood.pl 686 2010-12-16 00:02:50Z pro $ $URL: svn://svn.setun.net/dcppp/trunk/examples/flood.pl $
 
 =copyright
+
+NOT SUPPORTED, OLD
+
+
+
 flood tests
 
 flood.pl config:
@@ -11,11 +16,10 @@ run flood.pl dchub://1.4.5.6:4111
 
 
 =cut
-
 use strict;
 eval { use Time::HiRes qw(time sleep); };
 use Socket;
-use lib '../lib';
+use lib::abs '../lib';
 use Net::DirectConnect::clihub;
 our (%config);
 
@@ -129,10 +133,10 @@ TRY: for ( 0 .. $config{'flood_tries'} ) {
   #print("destroy1.\n"),
   handler( 'destroy', $dc ), next if !$dc->active();    #!$dc->{'socket'} and $dc->{'disconnect_recursive'};
   for ( 1 .. $config{'connect_wait'} ) {                #sleep(5); $dc->recv();
-    #last if (!$dc->{'socket'} and $dc->{'disconnect_recursive'}) or $dc->{'status'} eq 'connected';
+        #last if (!$dc->{'socket'} and $dc->{'disconnect_recursive'}) or $dc->{'status'} eq 'connected';
     last if ( !$dc->active() ) or $dc->{'status'} eq 'connected';
-    $dc->wait_sleep(10);                                #for 0 .. 10;
-    #sleep(1);
+    $dc->wait_sleep(10);    #for 0 .. 10;
+                            #sleep(1);
   }
   #print("destroy2.\n"),
   handler( 'destroy', $dc ), next if !$dc->active();    #!$dc->{'socket'} and $dc->{'disconnect_recursive'};
