@@ -1,4 +1,4 @@
-#$Id: adc.pm 686 2010-12-16 00:02:50Z pro $ $URL: svn://svn.setun.net/dcppp/trunk/lib/Net/DirectConnect/adc.pm $
+#$Id: adc.pm 706 2010-12-29 19:07:01Z pro $ $URL: svn://svn.setun.net/dcppp/trunk/lib/Net/DirectConnect/adc.pm $
 package    #hide from cpan
   Net::DirectConnect::adc;
 use strict;
@@ -12,8 +12,12 @@ use Net::DirectConnect;
 #use Net::DirectConnect::clicli;
 use Net::DirectConnect::http;
 #use Net::DirectConnect::httpcli;
+
+use lib::abs('pslib');
+use psmisc;    # REMOVE
+
 no warnings qw(uninitialized);
-our $VERSION = ( split( ' ', '$Revision: 686 $' ) )[1];
+our $VERSION = ( split( ' ', '$Revision: 706 $' ) )[1];
 use base 'Net::DirectConnect';
 our %codesSTA = (
   '00' => 'Generic, show description',
@@ -100,6 +104,7 @@ sub init {
     'port'     => 412,
     'host'     => 'localhost',
     'protocol' => 'adc',
+      'adc'     => 1,
     #'Pass' => '',
     #'key'  => 'zzz',
     #'auto_wait'        => 1,
@@ -109,6 +114,11 @@ sub init {
     #@_,
     'incomingclass' => __PACKAGE__,                        #'Net::DirectConnect::adc',
     no_print => { 'INF' => 1, 'QUI' => 1, 'SCH' => 1, },
+   'ID_file' => 'ID',
+
+      'cmd_bef' => undef,
+      'cmd_aft' => "\x0A",
+  
   );
   #$self->{$_} ||= $_{$_} for keys %_;
   !exists $self->{$_} ? $self->{$_} ||= $_{$_} : () for keys %_;
@@ -120,6 +130,9 @@ sub init {
   $self->{SUPAD}{H}{$_} = $_ for qw(BAS0 BASE TIGR UCM0 BLO0 BZIP );
   $self->{SUPAD}{I}{$_} = $_ for qw(BASE TIGR BZIP);
   $self->{SUPAD}{C}{$_} = $_ for qw(BASE TIGR BZIP);
+  if ( $self->{'broadcast'} ) {
+    $self->{SUPAD}{B} = $self->{SUPAD}{C};
+  }
   if ( $self->{'hub'} ) {
     $self->{'auto_connect'}         = 0;
     $self->{'auto_listen'}          = 1;
@@ -157,16 +170,38 @@ sub init {
   }
   $self->{hash_base} ||= sub { shift if ref $_[0]; $self->base_encode( $self->hash( $_[0] ) ) };
   #sub hash ($) { base32( tiger( $_[0] ) ); }
+
+  $self->{cmd_direct} ||= sub {
+    my $self = shift if ref $_[0];
+		my $peerid = shift;
+       local $self->{'host'} = $self->{'peers'}{$peerid}{'INF'}{I4},
+      local $self->{'port'} = $self->{'peers'}{$peerid}{'INF'}{U4} if $self->{'peers'}{$peerid}{'INF'}{I4} and $self->{'peers'}{$peerid}{'INF'}{U4};
+      $self->cmd( @_ );
+ };
+
+  
   $self->{INF_generate} ||= sub {
     my $self = shift if ref $_[0];
+    #$self->log( 'dev', 'ing_generate', $self->{'myport'},$self->{'myport_udp'}, $self->{'myip'});
+    #$self->{'clients'}{'listener_udp'}
+
+
     $self->{'INF'}{'NI'} ||= $self->{'Nick'} || 'perlAdcDev';
     $self->{'PID'} ||= MIME::Base32::decode $self->{'INF'}{'PD'} if $self->{'INF'}{'PD'};
     $self->{'CID'} ||= MIME::Base32::decode $self->{'INF'}{'ID'} if $self->{'INF'}{'ID'};
-    $self->{'ID'}  ||= 'perl' . $self->{'myip'} . $self->{'INF'}{'NI'};
+    if (-s $self->{'ID_file'}) {
+        $self->{'ID'}  ||= psmisc::file_read($self->{'ID_file'});
+    }
+    unless ($self->{'ID'}) {
+    	$self->{'ID'}  ||= join ' ', 'perl' , $self->{'myip'} ,  $VERSION , $0 ,$self->{'INF'}{'NI'}, time;
+    	psmisc::file_rewrite($self->{'ID_file'}, $self->{'ID'});
+    }
     $self->{'PID'} ||= $self->hash( $self->{'ID'} );
     $self->{'CID'} ||= $self->hash( $self->{'PID'} );
     $self->{'INF'}{'PD'} ||= $self->base_encode( $self->{'PID'} );
     $self->{'INF'}{'ID'} ||= $self->base_encode( $self->{'CID'} );
+    $self->{'INF'}{'SID'} ||= $self->{broadcast} ? $self->{'INF'}{'ID'} : substr $self->{'INF'}{'ID'}, 0, 4;
+    #sid
 #$self->log( 'id gen',"iID=$self->{'INF'}{'ID'} iPD=$self->{'INF'}{'PD'} PID=$self->{'PID'} CID=$self->{'CID'} ID=$self->{'ID'}" );
     $self->{'INF'}{'SL'} ||= $self->{'S'}         || '2';
     $self->{'INF'}{'SS'} ||= $self->{'sharesize'} || 20025693588;
@@ -177,19 +212,21 @@ sub init {
     $self->{'INF'}{'VE'} ||= $self->{'client'} . $self->{'V'}
       || 'perl'
       . $Net::DirectConnect::VERSION . '_'
-      . $VERSION;    #. '_' . ( split( ' ', '$Revision: 686 $' ) )[1];    #'++\s0.706';
+      . $VERSION;    #. '_' . ( split( ' ', '$Revision: 706 $' ) )[1];    #'++\s0.706';
     $self->{'INF'}{'US'} ||= 10000;
-    $self->{'INF'}{'U4'} ||= $self->{'myport_udp'};
+    $self->{'INF'}{'U4'} ||= $self->{'myport_udp'} || $self->{'myport'}; #maybe if broadcast only
     $self->{'INF'}{'I4'} ||= $self->{'myip'};
     $self->{'INF'}{'SU'} ||= 'ADC0,TCP4,UDP4';
     return $self->{'INF'};
   };
+$self->INF_generate();
+
   $self->{'parse'} ||= {
 #
 #=================
 #ADC dev
 #
-#'ISUP' => sub { }, 'ISID' => sub { $self->{'sid'} = $_[0] }, 'IINF' => sub { $self->cmd('BINF') },    'IQUI' => sub { },    'ISTA' => sub { $self->log( 'dcerr', @_ ) },
+#'ISUP' => sub { }, 'ISID' => sub { $self->{'INF'}{'SID'} = $_[0] }, 'IINF' => sub { $self->cmd('BINF') },    'IQUI' => sub { },    'ISTA' => sub { $self->log( 'dcerr', @_ ) },
     'SUP' => sub {
       my $self = shift if ref $_[0];
       my ( $dst, $peerid ) = @{ shift() };
@@ -213,6 +250,9 @@ sub init {
                                       #for keys %{$self->{'peers'}};
         $self->{'status'} = 'connected';
       }
+      elsif ( $dst eq 'C' ) {
+               $self->cmd( $dst, 'INF', ) unless $self->{count_sendcmd}{CINF};   
+      }
       $peerid ||= '';
       for ( $self->adc_strings_decode(@_) ) {
         if   ( (s/^(AD|RM)//)[0] eq 'RM' ) { delete $self->{'peers'}{$peerid}{'SUP'}{$_}; }
@@ -227,15 +267,20 @@ sub init {
         $self->{'peers'}{$peerid}{'SUP'}{ $params->{$_} } = 1 if $_ eq 'AD';
       }
 =cut      
+      #$self->log('adcdev', 'SUPans:', $peerid, $self->{'peers'}{$peerid}{'INF'}{I4}, $self->{'peers'}{$peerid}{'INF'}{U4});
 
-      $self->cmd( 'D', 'INF', ) if $self->{'broadcast'};
+      
+      #local $self->{'host'} = $self->{'peers'}{$peerid}{'INF'}{I4}; #can answer direct
+      #local $self->{'port'} = $self->{'peers'}{$peerid}{'INF'}{U4};
+      #$self->cmd( 'D', 'INF', ) if $self->{'broadcast'} and $self->{'broadcast_INF'};
+      #$self->cmd_direct( 'D', 'INF', ) if $self->{'broadcast'} and $self->{'broadcast_INF'};
       return $self->{'peers'}{$peerid}{'SUP'};
     },
     'SID' => sub {
       my $self = shift if ref $_[0];
-      $self->{'sid'} = $_[1];
-      $self->log( 'adcdev', 'SID:', $self->{'sid'} );
-      return $self->{'sid'};
+      $self->{'INF'}{'SID'} = $_[1];
+      $self->log( 'adcdev', 'SID:', $self->{'INF'}{'SID'} );
+      return $self->{'INF'}{'SID'};
     },
     'INF' => sub {
       my $self = shift if ref $_[0];
@@ -268,12 +313,24 @@ sub init {
       #$dst eq 'I' ?
       $self->log( 'adcdev', "ip change from [$params->{I4}] to [$self->{hostip}] " ), $params->{I4} = $self->{hostip}
         if $dst eq 'B' and $self->{parent}{hub} and $params->{I4} and $params->{I4} ne $self->{hostip};   #!$self->{parent}{hub}
-      if ( $dst eq 'B' and $self->{broadcast} ) {
-        $self->log( 'adcdev', "ip change from [$params->{I4}] to [$self->{recv_hostip}:$self->{recv_port}] " );
+      if ( #$dst eq 'B' and 
+      $self->{broadcast} ) {
+        $self->log( 'adcdev', "ip change from [$params->{I4}] to [$self->{recv_hostip}:$self->{recv_port}] ($self->{recv_hostip}:$self->{port})" );
         #$params->{U4} = $self->{recv_port};
-        $params->{U4} = $self->{port};
-        $params->{I4} = $self->{recv_hostip};
+        $params->{U4} ||= $self->{port};
+        $params->{I4} ||= $self->{recv_hostip};
+		
       }
+	  if ($peerid eq $self->{'INF'}{'SID'} and !$self->{myip}) {
+	  $self->{myip} ||= $params->{I4};
+	  $self->{'INF'}{'I4'} ||= $params->{I4};
+	    $self->log( 'adcdev', "ip  detected: [$self->{myip}:$self->{myport}]");
+	  
+	  }
+
+      my $first_seen;
+      $first_seen = 1 unless $self->{'peers'}{$peerid}{INF};
+      $self->log( 'adcdev',  "peer[$first_seen]: $peerid : $self->{'peers'}{$peerid}");
       $self->{'peers'}{$peerid}{'INF'}{$_} = $params->{$_} for keys %$params;
       $self->{'peers'}{$peerid}{'object'} = $self;
       $self->{'peers'}{ $params->{ID} }                              ||= $self->{'peers'}{$peerid};
@@ -287,7 +344,7 @@ sub init {
         $self->{'status'} = 'connected';    #clihub
       } elsif ( $dst eq 'C' ) {
         $self->{'status'} = 'connected';    #clicli
-        $self->cmd( $dst, 'INF' );
+        $self->cmd( $dst, 'INF' )unless $self->{count_sendcmd}{CINF};
         if   ( $params->{TO} ) { }
         else                   { }
         $self->cmd('file_select');
@@ -295,12 +352,22 @@ sub init {
       }
       #$self->log('adcdev', 'INF8', $peerid, @_);
       #if ($sendbinf) { $self->cmd( 'B', 'INF', $_, $self->{'peers_sid'}{$_}{'INF'} ) for keys %{ $self->{'peers_sid'} }; }
-      #$self->log('adcdev', 'INF9', $peerid, @_);
+      #$self->log('adcdev', 'INF9', $peerid, "H:$self->{parent}{hub}", @_);
       if ( $self->{parent}{hub} ) {
         my $params_send = \%$params;
         delete $params_send->{PD};
         $self->cmd_all( $dst, 'INF', $peerid, $self->adc_make_string($params_send) );
       }
+
+      $self->log('adcdev', "first_seen: $first_seen,$peerid ne $self->{'INF'}{'SID'}");
+
+      if($first_seen and 
+	  $self->{'broadcast'} and $peerid ne $self->{'INF'}{'SID'}) {
+
+            #$self->cmd( 'D', 'INF', ) if $self->{'broadcast'} and $self->{'broadcast_INF'};
+      $self->cmd_direct( $peerid, 'D', 'INF', ) if $self->{'broadcast'} and $self->{'broadcast_INF'};
+      }
+      
       return $params;    #$self->{'peers'}{$peerid}{'INF'};
     },
     'QUI' => sub {
@@ -352,7 +419,7 @@ sub init {
         $self->log( 'adcdev', 'SCH', ( $dst, $peerid, 'F=>', @feature ),
           $founded, -s $founded, -e $founded, 'c=', $self->{chrarset_fs}, );
         local @_ = (
-          $peerid, {
+          {
             SI => ( -s $founded ) || -1,
             SL => $self->{INF}{SL},
             FN => $self->adc_path_encode($foundedshow),
@@ -364,15 +431,15 @@ sub init {
           $self->log(
             'dcdev', 'SCH', 'i=', $self->{'peers'}{$peerid}{INF}{I4},
             'u=', $self->{'peers'}{$peerid}{INF}{U4},
-            'T==>', 'U' . 'RES' . $self->adc_make_string(@_)
+            'T==>', 'U' . 'RES ' . $self->adc_make_string($self->{'INF'}{'ID'}, @_)
           );
           $self->send_udp(
             $self->{'peers'}{$peerid}{INF}{I4},
             $self->{'peers'}{$peerid}{INF}{U4},
-            'U' . 'RES ' . $self->adc_make_string(@_)
+            'U' . 'RES ' .$self->adc_make_string($self->{'INF'}{'ID'}, @_) #. $self->{'cmd_aft'}
           );
         } else {
-          $self->cmd( 'D', 'RES', @_ );
+          $self->cmd( 'D', 'RES', $self->adc_make_string($peerid, @_) );
         }
       }
       #$self->adc_make_string(@_);
@@ -387,7 +454,7 @@ sub init {
       my $self = shift if ref $_[0];
       my ( $dst, $peerid, $toid ) = @{ shift() };
       #test $_[1] eq 'I'!
-      #$self->log( 'adcdev', '0RES:', "[d=$dst,p=$peerid,t=$toid]", join ':', @_ );
+      $self->log( 'adcdev', '0RES:', "[d=$dst,p=$peerid,t=$toid]", join ':', @_ );
       my $params = $self->adc_parse_named(@_);
       #$self->log('adcdev', 'RES:',"[d=$dst,p=$peerid]",Dumper $params);
       if ( $dst eq 'D' and $self->{'parent'}{'hub'} and ref $self->{'peers'}{$toid}{'object'} ) {
@@ -395,7 +462,12 @@ sub init {
       }
       if ( exists $self->{'want_download'}{ $params->{'TR'} } ) {
         $self->{'want_download'}{ $params->{'TR'} }{$peerid} = $params;    #maybe not all
-      }
+        if (my ($file) = $params->{FN} =~ m{([^\\/]+)$}) {
+          ++$self->{'want_download_filename'}{ $params->{TR} }{$file};
+		}
+		        $params->{CID} = $peerid;
+        $self->{'want_download'}{ $params->{TR} }{$peerid} = $params; # _tth_from
+		}
       $params;
     },
     'MSG' => sub {
@@ -411,14 +483,16 @@ sub init {
       my $self = shift if ref $_[0];
       my ( $dst, $peerid, $toid ) = @{ shift() };
       #$self->log( 'dcdev', "( $dst, RCM, $peerid, $toid )", @_ );
-      $self->cmd( $dst, 'CTM', $peerid, $_[0], $self->{'myport'}, $_[1], ) if $toid eq $self->{'sid'};
+      $self->cmd( $dst, 'CTM', $peerid, $_[0], $self->{'myport'}, $_[1], ) if $toid eq $self->{'INF'}{'SID'};
       if ( $dst eq 'D' and $self->{'parent'}{'hub'} and ref $self->{'peers'}{$toid}{'object'} ) {
         $self->{'peers'}{$toid}{'object'}->cmd( 'D', 'RCM', $peerid, $toid, @_ );
       }
-
 =z      
-       $self->{'clients'}{ $host . ':' . $port } = Net::DirectConnect::clicli->new(
-        %$self, $self->clear(),
+	my $host= $self->{'peers'}{$toid}{I4};
+	my $port= $self->{'peers'}{$toid}{U4}
+       $self->{'clients'}{ $host . ':' . $port } = __PACKAGE__->new(
+        #%$self, $self->clear(),
+        'parent' => $self,
         'host'         => $host,
         'port'         => $port,
 #'want'         => \%{ $self->{'want'} },
@@ -435,7 +509,7 @@ sub init {
       my ( $dst,   $peerid, $toid )  = @{ shift() };
       my ( $proto, $port,   $token ) = @_;
       my $host = $self->{'peers'}{$peerid}{'INF'}{'I4'};
-      $self->log( 'dcdev', "( $dst, CTM, $peerid, $toid ) - ($proto, $port, $token)", );
+      $self->log( 'dcdev', "( $dst, CTM, $peerid, $toid ) - ($proto, $port, $token) me=$self->{'INF'}{'SID'}", );
       $self->log( 'dcerr', 'CTM: unknown host', "( $dst, CTM, $peerid, $toid ) - ($proto, $port, $token)" ) unless $host;
       $self->{'clients'}{ $self->{'peers'}{$peerid}{'INF'}{ID} or $host . ':' . $port } = __PACKAGE__->new(
         #%$self, $self->clear(),
@@ -454,7 +528,7 @@ sub init {
         'auto_connect' => 1,
         'reconnects'   => 0,
         no_listen      => 1,
-      ) if $toid eq $self->{'sid'};
+      ) if $toid eq $self->{'INF'}{'SID'};
       if ( $dst eq 'D' and $self->{'parent'}{'hub'} and ref $self->{'peers'}{$toid}{'object'} ) {
         $self->{'peers'}{$toid}{'object'}->cmd( 'D', 'CTM', $peerid, $toid, @_ );
       }
@@ -463,7 +537,7 @@ sub init {
       my $self = shift if ref $_[0];
       my ( $dst, $peerid, $toid ) = @{ shift() };
       #CSND file files.xml.bz2 0 6117
-      $self->{'filetotal'} = $_[2] + $_[3];
+      $self->{'filetotal'} //= $_[2] + $_[3];
       return $self->file_open();
     },
     #CGET file TTH/YDIXOH7A3W233WTOQUET3JUGMHNBYNFZ4UBXGNY 637534208 6291456
@@ -542,9 +616,11 @@ sub init {
       #print "RUNADC![$self->{'protocol'}:$self->{'adc'}]";
       my $self = shift if ref $_[0];
       #$self->log($self, 'connect_aft inited',"MT:$self->{'message_type'}", ' ');
+      #{ 
+      $self->cmd( $self->{'message_type'}, 'SUP' ); 
+      #}
       if ( $self->{'broadcast'} ) { $self->cmd( $self->{'message_type'}, 'INF' ); }
       #else
-      { $self->cmd( $self->{'message_type'}, 'SUP' ); }
     },
     'cmd_all' => sub {
       my $self = shift if ref $_[0];
@@ -580,24 +656,24 @@ sub init {
         } elsif ( $dst eq 'B' ) {
           $self->cmd_adc    #sendcmd
             (
-            $dst, 'INF',    #$self->{'sid'},
+            $dst, 'INF',    #$self->{'INF'}{'SID'},
             @_,
             #map { $_ . $self->{'INF'}{$_} } $dst eq 'C' ? qw(ID TO) : sort keys %{ $self->{'INF'} }
             );
           return;
         }
       } else {
-        $self->INF_generate();
+        #$self->INF_generate();
      #$self->{''} ||= $self->{''} || '';
-     #$self->sendcmd( $dst, 'INF', $self->{'sid'}, map { $_ . $self->{$_} } grep { length $self->{$_} } @{ $self->{'BINFS'} } );
+     #$self->sendcmd( $dst, 'INF', $self->{'INF'}{'SID'}, map { $_ . $self->{$_} } grep { length $self->{$_} } @{ $self->{'BINFS'} } );
       }
       $self->cmd_adc        #sendcmd
         (
-        $dst, 'INF',        #$self->{'sid'},
-        map { $_ . $self->{'INF'}{$_} } $dst eq 'C' ? qw(ID TO) : @_ ? @_ : sort keys %{ $self->{'INF'} }
+        $dst, 'INF',        #$self->{'INF'}{'SID'},
+        map { $_ . $self->{'INF'}{$_} } grep {length $self->{'INF'}{$_}} $dst eq 'C' ? qw(ID TO) : @_ ? @_ : qw(ID I4 I6 U6 SS SF VE US DS SL AS AM EM NI HN HR HO TO CT SU RF)#sort keys %{ $self->{'INF'} }
         );
       #grep { length $self->{$_} } @{ $self->{'BINFS'} } );
-      #$self->cmd_adc( $dst, 'INF', $self->{'sid'}, map { $_ . $self->{$_} } grep { $self->{$_} } @{ $self->{'BINFS'} } );
+      #$self->cmd_adc( $dst, 'INF', $self->{'INF'}{'SID'}, map { $_ . $self->{$_} } grep { $self->{$_} } @{ $self->{'BINFS'} } );
       #BINF UUXX IDFXC3WTTDXHP7PLCCGZ6ZKBHRVAKBQ4KUINROXXI PDP26YAWX3HUNSTEXXYRGOIAAM2ZPMLD44HCWQEDY NI SL2 SS20025693588
       #SF30999 HN2 HR0 HO0 VE++\s0.706 US5242 SUADC0
     },
@@ -618,7 +694,7 @@ sub init {
       local %_;
       for my $w qw(SS SF) {
         #$self->log( 'dev', 'calc', $_, $w),
-        $_{$w} += $self->{'peers'}{$_}{INF}{$w} for grep { $_ and $_ ne $self->{sid} } keys %{ $self->{'peers_sid'} };
+        $_{$w} += $self->{'peers'}{$_}{INF}{$w} for grep { $_ and $_ ne $self->{'INF'}{'SID'} } keys %{ $self->{'peers_sid'} };
       }
       $_{UC} = keys %{ $self->{'peers'} };
       return \%_;
@@ -645,10 +721,14 @@ sub init {
       $self->cmd_adc( $dst, 'SND', @_ );
     },
 =cut    
-  #$self->log( 'dev', "0making listeners [$self->{'M'}]" );
+  $self->log( 'dev', "0making listeners [$self->{'M'}]:$self->{'no_listen'}" );
   unless ( $self->{'no_listen'} ) {
-    if ( ( $self->{'M'} eq 'A' or !$self->{'M'} ) and !$self->{'auto_listen'} and !$self->{'incoming'} ) {
-      $self->log( 'dev', "making listeners: tcp; class=", $self->{'incomingclass'} );
+      $self->log( 'dev', 'nyportgen',"$self->{'M'} eq 'A' or !$self->{'M'} ) and !$self->{'auto_listen'} and !$self->{'incoming'}");
+    if ( ( $self->{'M'} eq 'A' or !$self->{'M'} )  and !$self->{'incoming'} ) {
+if (!$self->{'auto_listen'} or #$self->{'Proto'} ne 'tcp'
+$self->{broadcast}
+) {     
+	 $self->log( 'dev', "making listeners: tcp; class=", $self->{'incomingclass'} );
       $self->{'clients'}{'listener_tcp'} = $self->{'incomingclass'}->new(
         #%$self, $self->clear(),
         #'want' => $self->{'want'},
@@ -662,6 +742,10 @@ sub init {
       );
       $self->{'myport'} = $self->{'myport_tcp'} = $self->{'clients'}{'listener_tcp'}{'myport'};
       $self->log( 'err', "cant listen tcp (file transfers)" ) unless $self->{'myport_tcp'};
+	  }
+if (!$self->{'auto_listen'} 
+#and $self->{'Proto'} ne 'udp'
+){
       $self->log( 'dev', "making listeners: udp" );
       $self->{'clients'}{'listener_udp'} = $self->{'incomingclass'}->new(
         #%$self, $self->clear(),
@@ -689,8 +773,10 @@ sub init {
         },
       );
       $self->{'myport_udp'} = $self->{'clients'}{'listener_udp'}{'myport'};
+      $self->log( 'dev', 'nyportgen',$self->{'myport_udp'});
       $self->log( 'err', "cant listen udp (search repiles)" ) unless $self->{'myport_udp'};
     }
+	}
     #DEV=z
 
 =no
@@ -735,7 +821,7 @@ $self->log( 'info', 'listening broadcast ', $self->{'dev_broadcast'} || $self->{
     delete $self->{'peers'}{ $self->{'peerid'} };
     delete $self->{'peers_sid'}{ $self->{'peerid'} };
     $self->cmd_all( 'I', 'QUI', $self->{'peerid'}, ) if $self->{'parent'}{'hub'};
-    delete $self->{'sid'};
+    delete $self->{'INF'}{'SID'};
     $self->log(
       'dev',  'disconnect int',           #psmisc::caller_trace(30)
       'hub=', $self->{'parent'}{'hub'},
